/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.awt.*;
import java.awt.event.*;

import jalview.datamodel.*;

public class AnnotationLabels
    extends Panel implements ActionListener
{
  boolean active = false;
  AlignmentPanel ap;
  boolean resizing = false;
  int oldY, mouseX;
  static String ADDNEW = "Add new row";
  static String HIDE = "Hide this row";
  static String DELETE = "Delete this row";
  static String SHOWALL = "Show all hidden rows";
  static String OUTPUT_TEXT = "Show Values In Textbox";
  int selectedRow = 0;
  int scrollOffset = 0;

  public AnnotationLabels(AlignmentPanel ap)
  {
    this.ap = ap;
    setLayout(null);
    addMouseListener(new MouseAdapter()
    {
      public void mousePressed(MouseEvent evt)
      {
        doMousePressed(evt);
      }
    });
  }

  public void setScrollOffset(int y)
  {
    scrollOffset = y;
    repaint();
  }

  public void actionPerformed(ActionEvent evt)
  {
    AlignmentAnnotation[] aa = ap.av.alignment.getAlignmentAnnotation();

    if (evt.getActionCommand().equals(HIDE))
    {
      aa[selectedRow].visible = false;
    }
    else if (evt.getActionCommand().equals(SHOWALL))
    {
      for (int i = 0; i < aa.length; i++)
      {
        aa[i].visible = true;
      }
    }
    else if (evt.getActionCommand().equals(OUTPUT_TEXT))
    {
      CutAndPasteTransfer cap = new CutAndPasteTransfer(false, ap.alignFrame.applet);
      Frame frame = new Frame();
      frame.add(cap);
      jalview.bin.JalviewLite.addFrame(frame,
                                       ap.alignFrame.getTitle() + " - " +
                                       aa[selectedRow].label, 500, 100);
      cap.setText(aa[selectedRow].toString());
    }

    ap.annotationPanel.adjustPanelHeight();
    setSize(getSize().width, ap.annotationPanel.getSize().height);
    ap.validate();
    ap.repaint();
  }

  public void doMousePressed(MouseEvent evt)
  {
    int y = evt.getY() - scrollOffset;
    AlignmentAnnotation[] aa = ap.av.alignment.getAlignmentAnnotation();
    int height = 0;
    for (int i = 0; i < aa.length; i++)
    {
      if (!aa[i].visible)
      {
        continue;
      }

      height += aa[i].height;
      if (y < height)
      {
        selectedRow = i;
        break;
      }
    }

    PopupMenu pop = new PopupMenu("Annotations");
    MenuItem item = new MenuItem(HIDE);
    item.addActionListener(this);
    pop.add(item);
    item = new MenuItem(SHOWALL);
    item.addActionListener(this);
    pop.add(item);
    this.add(pop);
    item = new MenuItem(OUTPUT_TEXT);
    item.addActionListener(this);
    pop.add(item);

    if (aa[selectedRow].label.equals("Consensus"))
    {
      pop.addSeparator();
      final CheckboxMenuItem cbmi = new CheckboxMenuItem(
          "Ignore Gaps In Consensus",
          ap.av.getIgnoreGapsConsensus());

      cbmi.addItemListener(new ItemListener()
      {
        public void itemStateChanged(ItemEvent e)
        {
          ap.av.setIgnoreGapsConsensus(cbmi.getState());
          ap.repaint();
        }
      });
      pop.add(cbmi);
    }

     pop.show(this, evt.getX(), evt.getY());

  }

  public void paint(Graphics g)
  {
    drawComponent(g);
  }

  public void drawComponent(Graphics g)
  {
    FontMetrics fm = g.getFontMetrics(g.getFont());
    g.setColor(Color.white);
    g.fillRect(0, 0, getSize().width, getSize().height);

    g.translate(0, scrollOffset);
    g.setColor(Color.black);

    AlignmentAnnotation[] aa = ap.av.alignment.getAlignmentAnnotation();
    int y = g.getFont().getSize();
    int x = 0;

    if (aa != null)
    {
      for (int i = 0; i < aa.length; i++)
      {
        if (!aa[i].visible)
        {
          continue;
        }

        x = getSize().width - fm.stringWidth(aa[i].label) - 3;

        if (aa[i].isGraph)
        {
          y += (aa[i].height / 3);
        }

        g.drawString(aa[i].label, x, y);

        if (aa[i].isGraph)
        {
          y += (2 * aa[i].height / 3);
        }
        else
        {
          y += aa[i].height;
        }
      }
    }
  }

}
