/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.util.*;

import java.awt.event.*;

import jalview.analysis.*;
import jalview.datamodel.*;
import jalview.io.*;
import jalview.jbappletgui.*;

public class TreePanel
    extends GTreePanel
{
  SequenceI[] seq;
  String type;
  String pwtype;
  AlignmentPanel ap;
  int start;
  int end;
  TreeCanvas treeCanvas;
  NJTree tree;

  public NJTree getTree()
  {
    return tree;
  }

  /**
   * Creates a new TreePanel object.
   *
   * @param av DOCUMENT ME!
   * @param seqVector DOCUMENT ME!
   * @param type DOCUMENT ME!
   * @param pwtype DOCUMENT ME!
   * @param s DOCUMENT ME!
   * @param e DOCUMENT ME!
   */
  public TreePanel(AlignViewport av, Vector seqVector, String type,
                   String pwtype, int s, int e)
  {
    super();
    initTreePanel(av, seqVector, type, pwtype, s, e, null);

    // We know this tree has distances. JBPNote TODO: prolly should add this as a userdefined default
    treeCanvas.setShowDistances(true);
  }

  /**
   * Creates a new TreePanel object.
   *
   * @param av DOCUMENT ME!
   * @param seqVector DOCUMENT ME!
   * @param newtree DOCUMENT ME!
   * @param type DOCUMENT ME!
   * @param pwtype DOCUMENT ME!
   */
  public TreePanel(AlignViewport av, Vector seqVector, NewickFile newtree,
                   String type, String pwtype)
  {
    super();
    initTreePanel(av, seqVector, type, pwtype, 0, seqVector.size(), newtree);
  }


  void initTreePanel(AlignViewport av, Vector seqVector, String type,
                     String pwtype, int s, int e, NewickFile newTree)
  {

    this.type = type;
    this.pwtype = pwtype;

    start = s;
    end = e;

    seq = new Sequence[seqVector.size()];
    for(int i=0; i<seqVector.size(); i++)
      seq[i] = (SequenceI)seqVector.elementAt(i);


    treeCanvas = new TreeCanvas(av, scrollPane);
    scrollPane.add(treeCanvas);


    TreeLoader tl = new TreeLoader(newTree);
    tl.start();

  }

  class TreeLoader extends Thread
  {
    NewickFile newtree;

    public TreeLoader(NewickFile newtree)
    {
      this.newtree = newtree;
    }

    public void run()
    {
      if(newtree!=null)
        tree = new NJTree(seq, newtree);
      else
        tree = new NJTree(seq, type, pwtype, start, end);

      tree.reCount(tree.getTopNode());
      tree.findHeight(tree.getTopNode());
      treeCanvas.setTree(tree);
      if(newtree!=null)
      {
        treeCanvas.setShowBootstrap(newtree.HasBootstrap());
        treeCanvas.setShowDistances(newtree.HasDistances());
      }


      treeCanvas.repaint();

    }
  }

  public String getText(String format)
  {
    return null;
  }

  protected void fitToWindow_actionPerformed(ActionEvent e)
  {
    treeCanvas.fitToWindow = fitToWindow.getState();
    repaint();
  }

  public void newickOutput_actionPerformed(ActionEvent actionEvent)
  {
    jalview.io.NewickFile fout = new jalview.io.NewickFile(tree.getTopNode());
    String output = fout.print(false, true);
    CutAndPasteTransfer cap = new CutAndPasteTransfer(false, ap.alignFrame.applet);
    cap.setText(output);
    java.awt.Frame frame = new java.awt.Frame();
    frame.add(cap);
    jalview.bin.JalviewLite.addFrame(frame, type + " " + pwtype, 500, 100);
  }

  protected void fontSize_actionPerformed(ActionEvent e)
  {
    /*   if( treeCanvas==null )
        return;

      String size = fontSize.getLabel().substring( fontSize.getLabel().indexOf("-")+1);

      Object selection = JOptionPane.showInternalInputDialog(Desktop.desktop,
                                  "Select font size",
                                  "Font size",
                                  JOptionPane.QUESTION_MESSAGE,
     null, new String[]{"1","2","4","6","8","10","12","14","16","18","20"}
                                  ,"Font Size - "+size);
      if(selection!=null)
      {
        fontSize.setText("Font Size - " + selection);

        int i = Integer.parseInt(selection.toString());
        treeCanvas.setFontSize(i);
      }
       scrollPane.setViewportView(treeCanvas);
     */
  }

  protected void distanceMenu_actionPerformed(ActionEvent e)
  {
    treeCanvas.setShowDistances(distanceMenu.getState());
  }

  protected void bootstrapMenu_actionPerformed(ActionEvent e)
  {
    treeCanvas.setShowBootstrap(bootstrapMenu.getState());
  }

  protected void placeholdersMenu_actionPerformed(ActionEvent e)
  {
    treeCanvas.setMarkPlaceholders(placeholdersMenu.getState());
  }

}
