/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.datamodel;

import jalview.analysis.PCA;

import jalview.io.*;

import jalview.jbgui.*;

import jalview.schemes.*;

import java.awt.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.5 $
 */
public class BinarySequence extends Sequence
{
    int[] binary;
    double[] dbinary;

    /**
     * Creates a new BinarySequence object.
     *
     * @param s DOCUMENT ME!
     */
    public BinarySequence(SequenceI s)
    {
        super(s.getName(), s.getSequence(), s.getStart(), s.getEnd());
    }

    /**
     * Creates a new BinarySequence object.
     *
     * @param name DOCUMENT ME!
     * @param sequence DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     */
    public BinarySequence(String name, String sequence, int start, int end)
    {
        super(name, sequence, start, end);
    }

    /**
     * DOCUMENT ME!
     */
    public void encode()
    {
        // Set all matrix to 0
        dbinary = new double[getSequence().length() * 21];

        int nores = 21;

        for (int i = 0; i < dbinary.length; i++)
        {
            dbinary[i] = 0.0;
        }

        for (int i = 0; i < getSequence().length(); i++)
        {
            int aanum = 20;

            try
            {
                aanum = ((Integer) ResidueProperties.getAAHash().get(getSequence()
                                                                         .substring(i,
                            i + 1))).intValue();
            }
            catch (NullPointerException e)
            {
                aanum = 20;
            }

            if (aanum > 20)
            {
                aanum = 20;
            }

            dbinary[(i * nores) + aanum] = 1.0;
        }
    }

    /**
     * DOCUMENT ME!
     */
    public void blosumEncode()
    {
        // Set all matrix to 0
        dbinary = new double[getSequence().length() * 21];

        int nores = 21;

        //for (int i = 0; i < dbinary.length; i++) {
        //  dbinary[i] = 0.0;
        //}
        for (int i = 0; i < getSequence().length(); i++)
        {
            int aanum = 20;

            try
            {
                aanum = ((Integer) ResidueProperties.getAAHash().get(getSequence()
                                                                         .substring(i,
                            i + 1))).intValue();
            }
            catch (NullPointerException e)
            {
                aanum = 20;
            }

            if (aanum > 20)
            {
                aanum = 20;
            }

            // Do the blosum thing
            for (int j = 0; j < 20; j++)
            {
                dbinary[(i * nores) + j] = ResidueProperties.getBLOSUM62()[aanum][j];
            }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String toBinaryString()
    {
        String out = "";

        for (int i = 0; i < binary.length; i++)
        {
            out += (new Integer(binary[i])).toString();

            if (i < (binary.length - 1))
            {
                out += " ";
            }
        }

        return out;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public double[] getDBinary()
    {
        return dbinary;
    }

    /**
     * DOCUMENT ME!
     *
     * @param rt DOCUMENT ME!
     */
    public static void printMemory(Runtime rt)
    {
        System.out.println("DEBUG: Free memory = " + rt.freeMemory()); // log.
    }
}
