/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.datamodel;

import jalview.analysis.*;

import java.awt.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.17 $
 */
public class Sequence implements SequenceI
{
    protected String name;
    protected String sequence;
    protected String description;
    protected int start;
    protected int end;
    protected String displayId;
    protected Color color = Color.white;
    String pdbId;

    /** DOCUMENT ME!! */
    public Vector sequenceFeatures = new Vector();

    /**
     * Creates a new Sequence object.
     *
     * @param name DOCUMENT ME!
     * @param sequence DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     */
    public Sequence(String name, String sequence, int start, int end)
    {
      this.name = name;
      this.sequence = sequence;
      this.start = start;
      this.end = end;

      if (end < 1)
      {
        int endRes = 0;
        char ch;
        for (int j = 0; j < sequence.length(); j++)
        {
          ch = sequence.charAt(j);
          if (!jalview.util.Comparison.isGap( (ch)))
          {
            endRes++;
          }
        }
        if (endRes > 0)
        {
          endRes += start - 1;
        }

        this.end = endRes;
      }

        setDisplayId();
    }

    /**
     * Creates a new Sequence object.
     *
     * @param name DOCUMENT ME!
     * @param sequence DOCUMENT ME!
     */
    public Sequence(String name, String sequence)
    {
        this(name, sequence, 1, sequence.length());
    }

    /**
     * Creates a new Sequence object.
     *
     * @param seq DOCUMENT ME!
     */
    public Sequence(SequenceI seq)
    {
        this(seq.getName(), seq.getSequence(), seq.getStart(), seq.getEnd());
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     */
    public void setSequenceFeatures(Vector v)
    {
        sequenceFeatures = v;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Vector getSequenceFeatures()
    {
        return sequenceFeatures;
    }

    /**
     * DOCUMENT ME!
     *
     * @param id DOCUMENT ME!
     */
    public void setPDBId(String id)
    {
        pdbId = id;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getPDBId()
    {
        return pdbId;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getDisplayId()
    {
        return displayId;
    }

    /**
     * DOCUMENT ME!
     */
    public void setDisplayId()
    {
        displayId = name + "/" + start + "-" + end;
    }

    /**
     * DOCUMENT ME!
     *
     * @param name DOCUMENT ME!
     */
    public void setName(String name)
    {
        this.name = name;
        setDisplayId();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getName()
    {
        return this.name;
    }

    /**
     * DOCUMENT ME!
     *
     * @param start DOCUMENT ME!
     */
    public void setStart(int start)
    {
        this.start = start;
        setDisplayId();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getStart()
    {
        return this.start;
    }

    /**
     * DOCUMENT ME!
     *
     * @param end DOCUMENT ME!
     */
    public void setEnd(int end)
    {
        this.end = end;
        setDisplayId();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getEnd()
    {
        return this.end;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getLength()
    {
        return this.sequence.length();
    }

    /**
     * DOCUMENT ME!
     *
     * @param seq DOCUMENT ME!
     */
    public void setSequence(String seq)
    {
        this.sequence = seq;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getSequence()
    {
        return this.sequence;
    }

    /**
     * DOCUMENT ME!
     *
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getSequence(int start, int end)
    {
        // JBPNote - left to user to pad the result here (TODO:Decide on this policy)
        if (start >= sequence.length())
        {
            return "";
        }

        if (end >= sequence.length())
        {
            end = sequence.length();
        }

        return this.sequence.substring(start, end);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public char getCharAt(int i)
    {
        if (i < sequence.length())
        {
            return sequence.charAt(i);
        }
        else
        {
            return ' ';
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param desc DOCUMENT ME!
     */
    public void setDescription(String desc)
    {
        this.description = desc;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getDescription()
    {
        return this.description;
    }

    /**
     * DOCUMENT ME!
     *
     * @param pos DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int findIndex(int pos)
    {
        // returns the alignment position for a residue
        int j = start;
        int i = 0;

        while ((i < sequence.length()) && (j <= end) && (j <= pos))
        {
            char c = sequence.charAt(i);

            if (!jalview.util.Comparison.isGap((c)))
            {
                j++;
            }

            i++;
        }

        if ((j == end) && (j < pos))
        {
            return end + 1;
        }
        else
        {
            return i;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int findPosition(int i)
    {
        // Returns the sequence position for an alignment position
        int j = 0;
        int pos = start;

        while ((j < i) && (j < sequence.length()))
        {
            char c = sequence.charAt(j);

            if (!jalview.util.Comparison.isGap((c)))
            {
                pos++;
            }

            j++;
        }

        return pos;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int[] gapMap()
    {
        // Returns an int array giving the position of each residue in the sequence in the alignment
        String seq = jalview.analysis.AlignSeq.extractGaps("-. ", sequence);
        int[] map = new int[seq.length()];
        int j = 0;
        int p = 0;

        while (j < sequence.length())
        {
            if (!jalview.util.Comparison.isGap(sequence.charAt(j)))
            {
                map[p++] = j;
            }

            j++;
        }

        return map;
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     */
    public void deleteCharAt(int i)
    {
        if (i >= sequence.length())
        {
            return;
        }

        sequence = sequence.substring(0, i) + sequence.substring(i + 1);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     * @param j DOCUMENT ME!
     */
    public void deleteChars(int i, int j)
    {
        if (i >= sequence.length())
        {
            return;
        }

        if (j >= sequence.length())
        {
            sequence = sequence.substring(0, i);
        }
        else
        {
            sequence = sequence.substring(0, i) + sequence.substring(j);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     * @param c DOCUMENT ME!
     */
    public void insertCharAt(int i, char c)
    {
        insertCharAt(i, c, true);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     * @param c DOCUMENT ME!
     * @param chop DOCUMENT ME!
     */
    public void insertCharAt(int i, char c, boolean chop)
    {
        String tmp = new String(sequence);

        if (i < sequence.length())
        {
            sequence = tmp.substring(0, i) + String.valueOf(c) +
                tmp.substring(i);
        }
        else
        {
            // JBPNote : padding char at end of sequence. We'll not get away with this when we insert residues, I bet!
            char[] ch = new char[(1 + i) - sequence.length()];

            for (int j = 0, k = ch.length; j < k; j++)
                ch[j] = c;

            sequence = tmp + String.valueOf(ch);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param c DOCUMENT ME!
     */
    public void setColor(Color c)
    {
        this.color = c;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Color getColor()
    {
        return color;
    }
}
