/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.datamodel.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import java.util.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.25 $
 */
public class AnnotationPanel extends JPanel implements MouseListener,
    MouseMotionListener, ActionListener, AdjustmentListener
{
    static String HELIX = "Helix";
    static String SHEET = "Sheet";
    static String LABEL = "Label";
    static String REMOVE = "Remove Annotation";
    static String COLOUR = "Colour";
    static Color HELIX_COLOUR = Color.red.darker();
    static Color SHEET_COLOUR = Color.green.darker().darker();

    /** DOCUMENT ME!! */
    public static int GRAPH_HEIGHT = 40;
    AlignViewport av;
    AlignmentPanel ap;
    int activeRow = -1;
    Vector activeRes;
    BufferedImage image;
    Graphics2D gg;
    FontMetrics fm;
    int imgWidth = 0;
    boolean fastPaint = false;

    /**
     * Creates a new AnnotationPanel object.
     *
     * @param ap DOCUMENT ME!
     */
    public AnnotationPanel(AlignmentPanel ap)
    {
        ToolTipManager.sharedInstance().registerComponent(this);
        ToolTipManager.sharedInstance().setInitialDelay(0);
        ToolTipManager.sharedInstance().setDismissDelay(10000);
        this.ap = ap;
        av = ap.av;
        this.setLayout(null);
        addMouseListener(this);
        addMouseMotionListener(this);
        ap.annotationScroller.getVerticalScrollBar().addAdjustmentListener(this);
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void adjustmentValueChanged(AdjustmentEvent evt)
    {
        ap.alabels.setScrollOffset(-evt.getValue());
    }

    /**
     * DOCUMENT ME!
     */
    public void adjustPanelHeight()
    {
        // setHeight of panels
        image = null;
        AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
        int height = 0;

        if (aa != null)
        {
            for (int i = 0; i < aa.length; i++)
            {
                if (!aa[i].visible)
                {
                    continue;
                }

                aa[i].height = 0;

                if (aa[i].hasText)
                {
                    aa[i].height += av.charHeight;
                }

                if (aa[i].hasIcons)
                {
                    aa[i].height += 16;
                }

                if (aa[i].isGraph)
                {
                    aa[i].height += GRAPH_HEIGHT;
                }

                if (aa[i].height == 0)
                {
                    aa[i].height = 20;
                }

                height += aa[i].height;
            }
        }
        else
        {
            height = 20;
        }

        this.setPreferredSize(new Dimension(1, height));
    }

    /**
     * DOCUMENT ME!
     *
     * @param col DOCUMENT ME!
     */
    public void removeEditableColumn(int col)
    {
        if (activeRow == -1)
        {
            AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
            if(aa==null)
              return;

            for (int j = 0; j < aa.length; j++)
            {
                if (aa[j].editable)
                {
                    activeRow = j;

                    break;
                }
            }
        }

        if ((activeRes != null) && activeRes.contains(String.valueOf(col)))
        {
            activeRes.removeElement(String.valueOf(col));
        }

        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param col DOCUMENT ME!
     */
    public void addEditableColumn(int col)
    {
        if (activeRow == -1)
        {
            AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
            if(aa==null)
              return;

            for (int j = 0; j < aa.length; j++)
            {
                if (aa[j].editable)
                {
                    activeRow = j;

                    break;
                }
            }
        }

        if (activeRes == null)
        {
            activeRes = new Vector();
        }

        if (!activeRes.contains(String.valueOf(col)))
        {
            activeRes.addElement(String.valueOf(col));
        }

        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void actionPerformed(ActionEvent evt)
    {
        AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
        Annotation[] anot = aa[activeRow].annotations;

        if (evt.getActionCommand().equals(REMOVE))
        {
            for (int i = 0; i < activeRes.size(); i++)
            {
                anot[Integer.parseInt(activeRes.get(i).toString())] = null;
                anot[Integer.parseInt(activeRes.get(i).toString())] = null;
            }
        }
        else if (evt.getActionCommand().equals(LABEL))
        {
            String label = JOptionPane.showInputDialog(this, "Enter Label ",
                    "Enter label", JOptionPane.QUESTION_MESSAGE);

            if (label == null)
            {
                return;
            }

            if ((label.length() > 0) && !aa[activeRow].hasText)
            {
                aa[activeRow].hasText = true;
            }

            for (int i = 0; i < activeRes.size(); i++)
            {
                int index = Integer.parseInt(activeRes.get(i).toString());

                if (anot[index] == null)
                {
                    anot[index] = new Annotation(label, "", ' ', 0);
                }

                anot[index].displayCharacter = label;
            }
        }
        else if (evt.getActionCommand().equals(COLOUR))
        {
            Color col = JColorChooser.showDialog(this,
                    "Choose foreground colour", Color.black);

            for (int i = 0; i < activeRes.size(); i++)
            {
                int index = Integer.parseInt(activeRes.get(i).toString());

                if (anot[index] == null)
                {
                    anot[index] = new Annotation("", "", ' ', 0);
                }

                anot[index].colour = col;
            }
        }
        else // HELIX OR SHEET
        {
            char type = 0;
            String symbol = "\u03B1";

            if (evt.getActionCommand().equals(HELIX))
            {
                type = 'H';
            }
            else if (evt.getActionCommand().equals(SHEET))
            {
                type = 'E';
                symbol = "\u03B2";
            }

            if (!aa[activeRow].hasIcons)
            {
                aa[activeRow].hasIcons = true;
            }

            String label = JOptionPane.showInputDialog("Enter a label for the structure?",
                    symbol);

            if (label == null)
            {
                return;
            }

            if ((label.length() > 0) && !aa[activeRow].hasText)
            {
                aa[activeRow].hasText = true;
            }

            for (int i = 0; i < activeRes.size(); i++)
            {
                int index = Integer.parseInt(activeRes.get(i).toString());

                if (anot[index] == null)
                {
                    anot[index] = new Annotation(label, "", type, 0);
                }

                anot[index].secondaryStructure = type;
                anot[index].displayCharacter = label;
            }
        }

        adjustPanelHeight();
        activeRes = null;
        repaint();

        return;
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mousePressed(MouseEvent evt)
    {
        int height = 0;
        activeRow = -1;

        AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();

        for (int i = 0; i < aa.length; i++)
        {
            if (aa[i].visible)
            {
              height += aa[i].height;
            }

            if (evt.getY() < height)
            {
                if (aa[i].editable)
                {
                    activeRow = i;
                }
                else
                {
                    activeRes = null;
                }

                break;
            }
        }

        if (SwingUtilities.isRightMouseButton(evt))
        {
            if (activeRes == null)
            {
                return;
            }

            JPopupMenu pop = new JPopupMenu("Structure type");
            JMenuItem item = new JMenuItem(HELIX);
            item.addActionListener(this);
            pop.add(item);
            item = new JMenuItem(SHEET);
            item.addActionListener(this);
            pop.add(item);
            item = new JMenuItem(LABEL);
            item.addActionListener(this);
            pop.add(item);
            item = new JMenuItem(COLOUR);
            item.addActionListener(this);
            pop.add(item);
            item = new JMenuItem(REMOVE);
            item.addActionListener(this);
            pop.add(item);
            pop.show(this, evt.getX(), evt.getY());

            return;
        }

        if (aa == null)
        {
            return;
        }

        int res = (evt.getX() / av.getCharWidth()) + av.getStartRes();

        if (evt.isControlDown() || evt.isAltDown())
        {
            addEditableColumn(res);
        }
        else if (evt.isShiftDown())
        {
            if (activeRes == null)
            {
                activeRes = new Vector();
            }
            else
            {
                int start = Integer.parseInt(activeRes.get(activeRes.size() -
                            1).toString());
                int end = res;

                if (end < start)
                {
                    int temp = end;
                    end = start;
                    start = temp;
                }

                for (int n = start; n <= end; n++)
                {
                    addEditableColumn(n);
                }
            }
        }
        else
        {
            activeRes = new Vector();
            activeRes.addElement(String.valueOf(res));
        }

        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mouseReleased(MouseEvent evt)
    {
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mouseEntered(MouseEvent evt)
    {
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mouseExited(MouseEvent evt)
    {
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mouseDragged(MouseEvent evt)
    {
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mouseMoved(MouseEvent evt)
    {
        AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();

        if (aa == null)
        {
            return;
        }

        int row = -1;
        int height = 0;

        for (int i = 0; i < aa.length; i++)
        {
            if (aa[i].visible)
            {
                height += aa[i].height;
            }

            if (evt.getY() < height)
            {
                row = i;

                break;
            }
        }

        int res = (evt.getX() / av.getCharWidth()) + av.getStartRes();

        if ((row > -1) && (res < aa[row].annotations.length) &&
                (aa[row].annotations[res] != null))
        {
            this.setToolTipText(aa[row].annotations[res].description);

            StringBuffer text = new StringBuffer("Sequence position " +
                    (res + 1) + "  " + aa[row].annotations[res].description);
            ap.alignFrame.statusBar.setText(text.toString());
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param evt DOCUMENT ME!
     */
    public void mouseClicked(MouseEvent evt)
    {
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     */
    public void paintComponent(Graphics g)
    {
      g.setColor(Color.white);
      g.fillRect(0, 0, getWidth(), getHeight());

      if (fastPaint)
      {
        g.drawImage(image, 0, 0, this);
        fastPaint = false;
        return;
      }

      imgWidth = (av.endRes - av.startRes + 1) * av.charWidth;

      if (image == null || imgWidth != image.getWidth())
      {
        image = new BufferedImage(imgWidth, ap.annotationPanel.getHeight(),
                                  BufferedImage.TYPE_INT_RGB);
        gg = (Graphics2D) image.getGraphics();
        gg.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);

        gg.setFont(av.getFont());
        fm = gg.getFontMetrics();
      }

      drawComponent(gg, av.startRes, av.endRes + 1);
      g.drawImage(image, 0, 0, this);
    }

    /**
     * DOCUMENT ME!
     *
     * @param horizontal DOCUMENT ME!
     */
    public void fastPaint(int horizontal)
    {
        if ((horizontal == 0) ||
                (av.alignment.getAlignmentAnnotation() == null) ||
                (av.alignment.getAlignmentAnnotation().length < 1))
        {
            repaint();

            return;
        }

        gg.copyArea(0, 0, imgWidth, getHeight(), -horizontal * av.charWidth, 0);

        int sr = av.startRes;
        int er = av.endRes + 1;
        int transX = 0;

        if (horizontal > 0) // scrollbar pulled right, image to the left
        {
            transX = (er - sr - horizontal) * av.charWidth;
            sr = er - horizontal;
        }
        else if (horizontal < 0)
        {
            er = sr - horizontal;
        }

        gg.translate(transX, 0);

        drawComponent(gg, sr, er);

        gg.translate(-transX, 0);

        fastPaint = true;

        repaint();

    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param startRes DOCUMENT ME!
     * @param endRes DOCUMENT ME!
     */
    public void drawComponent(Graphics2D g, int startRes, int endRes)
    {
        g.setColor(Color.white);
        g.fillRect(0, 0, (endRes - startRes) * av.charWidth, getHeight());

        if ((av.alignment.getAlignmentAnnotation() == null) ||
                (av.alignment.getAlignmentAnnotation().length < 1))
        {
            g.setColor(Color.white);
            g.fillRect(0, 0, getWidth(), getHeight());
            g.setColor(Color.black);
            g.drawString("Alignment has no annotations", 20, 15);

            return;
        }

        AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();

        int j;
        int x = 0;
        int y = 0;
        char[] lastSS = new char[aa.length];
        int[] lastSSX = new int[aa.length];
        int iconOffset = av.charHeight / 2;
        boolean validRes = false;

        //\u03B2 \u03B1
        for (int i = 0; i < aa.length; i++)
        {
            AlignmentAnnotation row = aa[i];

            if (!row.visible)
            {
                continue;
            }


            if (row.isGraph)
            {
                // this is so that we draw the characters below the graph
                y += row.height;

                if (row.hasText)
                {
                    y -= av.charHeight;
                }
            }

            if (row.hasText)
            {
                iconOffset = av.charHeight / 2;
            }
            else
            {
                iconOffset = 0;
            }

            for (j = startRes; j < endRes; j++)
            {
                if ((row.annotations.length <= j) ||
                        (row.annotations[j] == null))
                {
                    validRes = false;
                }
                else
                {
                    validRes = true;
                }

                x = (j - startRes) * av.charWidth;

                if (activeRow == i)
                {
                    g.setColor(Color.red);

                    if (activeRes != null)
                    {
                        for (int n = 0; n < activeRes.size(); n++)
                        {
                            int v = Integer.parseInt(activeRes.get(n).toString());

                            if (v == j)
                            {
                                g.fillRect((j - startRes) * av.charWidth, y,
                                    av.charWidth, row.height);
                            }
                        }
                    }
                }

                if (validRes &&
                        (row.annotations[j].displayCharacter.length() > 0))
                {

                    int charOffset = (av.charWidth -
                        fm.charWidth(row.annotations[j].displayCharacter.charAt(
                                0))) / 2;
                    g.setColor(row.annotations[j].colour);

                    if (j == 0 || row.isGraph)
                    {
                        g.drawString(row.annotations[j].displayCharacter, x+charOffset,
                            y + iconOffset + 3);
                    }
                    else if (((row.annotations[j - 1] == null) ||
                            (row.annotations[j].displayCharacter != row.annotations[j -
                            1].displayCharacter)))
                    {
                        g.drawString(row.annotations[j].displayCharacter, x+charOffset,
                            y + iconOffset + 3);
                    }
                }

                if (row.hasIcons)
                {
                    if (!validRes ||
                            (row.annotations[j].secondaryStructure != lastSS[i]))
                    {
                        switch (lastSS[i])
                        {
                        case 'H':
                            g.setColor(HELIX_COLOUR);
                            g.fillRoundRect(lastSSX[i], y + 4 + iconOffset,
                                x - lastSSX[i], 7, 8, 8);

                            break;

                        case 'E':
                            g.setColor(SHEET_COLOUR);
                            g.fillRect(lastSSX[i], y + 4 + iconOffset,
                                x - lastSSX[i] - 4, 7);
                            g.fillPolygon(new int[] { x - 4, x - 4, x },
                                new int[]
                                {
                                    y + iconOffset, y + 14 + iconOffset,
                                    y + 8 + iconOffset
                                }, 3);

                            break;

                        case 'C':
                            break;

                        default:
                            g.setColor(Color.gray);
                            g.fillRect(lastSSX[i], y + 6 + iconOffset,
                                x - lastSSX[i], 2);

                            break;
                        }

                        if (validRes)
                        {
                            lastSS[i] = row.annotations[j].secondaryStructure;
                        }
                        else
                        {
                            lastSS[i] = ' ';
                        }

                        lastSSX[i] = x;
                    }
                }

                if (validRes && row.isGraph)
                {
                    g.setColor(new Color(0, 0, 180));

                    int height = (int) ((row.annotations[j].value / row.graphMax) * GRAPH_HEIGHT);

                    if (row.windowLength > 1)
                    {
                        int total = 0;

                        for (int i2 = j - (row.windowLength / 2);
                                i2 < (j + (row.windowLength / 2)); i2++)
                        {
                            if ((i2 < 0) || (i2 >= av.alignment.getWidth()))
                            {
                                continue;
                            }

                            total += row.annotations[i2].value;
                        }

                        total /= row.windowLength;
                        height = (int) ((total / row.graphMax) * GRAPH_HEIGHT);
                    }

                    g.setColor(row.annotations[j].colour);
                    g.fillRect(x, y - height, av.charWidth, height);
                }
            }

            x += av.charWidth;

            if (row.hasIcons)
            {
                switch (lastSS[i])
                {
                case 'H':
                    g.setColor(HELIX_COLOUR);
                    g.fillRoundRect(lastSSX[i], y + 4 + iconOffset,
                        x - lastSSX[i], 7, 8, 8);

                    break;

                case 'E':
                    g.setColor(SHEET_COLOUR);
                    g.fillRect(lastSSX[i], y + 4 + iconOffset,
                        x - lastSSX[i] - 4, 7);
                    g.fillPolygon(new int[] { x - 4, x - 4, x },
                        new int[]
                        {
                            y + iconOffset, y + 14 + iconOffset,
                            y + 7 + iconOffset
                        }, 3);

                    break;

                case 'C':
                    break;

                default:
                    g.setColor(Color.gray);
                    g.fillRect(lastSSX[i], y + 6 + iconOffset, x - lastSSX[i], 2);

                    break;
                }
            }

            if (row.isGraph && row.hasText)
            {
                y += av.charHeight;
            }

            if (!row.isGraph)
            {
                y += aa[i].height;
            }
        }
    }

    // used by overview window
    public void drawGraph(Graphics g, AlignmentAnnotation aa, int width, int y, int sRes, int eRes)
    {
        g.setColor(Color.white);
        g.fillRect(0, 0, width, y);
        g.setColor(new Color(0, 0, 180));


        int x = 0, height;

        for (int j = sRes; j < eRes; j++)
        {
            g.setColor(new Color(0, 0, 180));

            height = (int) ((aa.annotations[j].value / aa.graphMax) * GRAPH_HEIGHT);
            if(height>y)
              height = y;

            g.fillRect(x, y - height, av.charWidth, height);
            x += av.charWidth;
        }
    }
}
