/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.bin.*;

import jalview.jbgui.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.12 $
 */
public class FontChooser extends GFontChooser
{
    AlignmentPanel ap;
    TreePanel tp;
    Font oldFont;
    boolean init = true;
    JInternalFrame frame;

    /**
     * Creates a new FontChooser object.
     *
     * @param ap DOCUMENT ME!
     */
    public FontChooser(TreePanel tp)
    {
      this.tp = tp;
      oldFont = tp.getTreeFont();
      defaultButton.setVisible(false);
      init();
    }
    /**
     * Creates a new FontChooser object.
     *
     * @param ap DOCUMENT ME!
     */
    public FontChooser(AlignmentPanel ap)
    {
      oldFont = ap.av.getFont();
      this.ap = ap;
      init();
    }

    void init()
    {
        frame = new JInternalFrame();
        frame.setContentPane(this);
        if(tp!=null)
           Desktop.addInternalFrame(frame, "Change Font (Tree Panel)", 480, 100, false);
        else
           Desktop.addInternalFrame(frame, "Change Font", 480, 100, false);

        frame.setLayer(JLayeredPane.PALETTE_LAYER);

        String[] fonts = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment()
                                                     .getAvailableFontFamilyNames();

        for (int i = 0; i < fonts.length; i++)
        {
            fontName.addItem(fonts[i]);
        }

        for (int i = 1; i < 31; i++)
        {
            fontSize.addItem(i + "");
        }

        fontStyle.addItem("plain");
        fontStyle.addItem("bold");
        fontStyle.addItem("italic");

        fontName.setSelectedItem(oldFont.getName());
        fontSize.setSelectedItem(oldFont.getSize() + "");
        fontStyle.setSelectedIndex(oldFont.getStyle());

        init = false;
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void ok_actionPerformed(ActionEvent e)
    {
        try
        {
            frame.setClosed(true);
        }
        catch (Exception ex)
        {
        }

        if(ap!=null)
        {
          if (ap.getOverviewPanel() != null)
          {
            ap.getOverviewPanel().updateOverviewImage();
          }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void cancel_actionPerformed(ActionEvent e)
    {
        if(ap!=null)
        {
          ap.av.setFont(oldFont);
          ap.repaint();
        }
        else if(tp!=null)
        {
          tp.setTreeFont(oldFont);
        }
        fontName.setSelectedItem(oldFont.getName());
        fontSize.setSelectedItem(oldFont.getSize() + "");
        fontStyle.setSelectedIndex(oldFont.getStyle());

        try
        {
            frame.setClosed(true);
        }
        catch (Exception ex)
        {
        }
    }

    /**
     * DOCUMENT ME!
     */
    void changeFont()
    {
      Font newFont = new Font(fontName.getSelectedItem().toString(),
                              fontStyle.getSelectedIndex(),
                              Integer.parseInt(fontSize.getSelectedItem().toString()));
      if (ap != null)
      {
        ap.av.setFont(newFont);
        ap.fontChanged();
      }
      else if(tp != null)
      {
        tp.setTreeFont(newFont);
      }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void fontName_actionPerformed(ActionEvent e)
    {
        if (init)
        {
            return;
        }

        changeFont();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void fontSize_actionPerformed(ActionEvent e)
    {
        if (init)
        {
            return;
        }

        changeFont();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void fontStyle_actionPerformed(ActionEvent e)
    {
        if (init)
        {
            return;
        }

        changeFont();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void defaultButton_actionPerformed(ActionEvent e)
    {
        Cache.setProperty("FONT_NAME", fontName.getSelectedItem().toString());
        Cache.setProperty("FONT_STYLE", fontStyle.getSelectedIndex() + "");
        Cache.setProperty("FONT_SIZE", fontSize.getSelectedItem().toString());
    }
}
