/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.analysis.*;

import jalview.datamodel.*;

import java.awt.*;
import java.awt.image.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.28 $
 */
public class IdCanvas extends JPanel
{
    protected AlignViewport av;
    protected boolean showScores = true;
    protected int maxIdLength = -1;
    protected String maxIdStr = null;
    BufferedImage image;
    Graphics2D gg;
    int imgHeight = 0;
    boolean fastPaint = false;
    java.util.Vector searchResults;

    /**
     * Creates a new IdCanvas object.
     *
     * @param av DOCUMENT ME!
     */
    public IdCanvas(AlignViewport av)
    {
        setLayout(new BorderLayout());
        this.av = av;
        PaintRefresher.Register(this, av.alignment);
    }

    /**
     * DOCUMENT ME!
     *
     * @param gg DOCUMENT ME!
     * @param s DOCUMENT ME!
     * @param i DOCUMENT ME!
     * @param starty DOCUMENT ME!
     * @param ypos DOCUMENT ME!
     */
    public void drawIdString(Graphics2D gg, SequenceI s, int i, int starty,
        int ypos)
    {
        int charHeight = av.charHeight;

        if ((searchResults != null) && searchResults.contains(s))
        {
            gg.setColor(Color.black);
            gg.fillRect(0, ((i - starty) * charHeight) + ypos, getWidth(),
                charHeight);
            gg.setColor(Color.white);
        }
        else if ((av.getSelectionGroup() != null) &&
                av.getSelectionGroup().sequences.contains(s))
        {
            gg.setColor(Color.lightGray);
            gg.fillRect(0, ((i - starty) * charHeight) + ypos, getWidth(),
                charHeight);
            gg.setColor(Color.white);
        }
        else
        {
            gg.setColor(s.getColor());
            gg.fillRect(0, ((i - starty) * charHeight) + ypos, getWidth(),
                charHeight);
            gg.setColor(Color.black);
        }

        String string = s.getName();

        if (av.getShowFullId())
        {
            string = s.getDisplayId();
        }

        gg.drawString(string, 0,
            (((i - starty + 1) * charHeight) + ypos) - (charHeight / 5));
    }

    /**
     * DOCUMENT ME!
     *
     * @param vertical DOCUMENT ME!
     */
    public void fastPaint(int vertical)
    {
        if (gg == null)
        {
            repaint();

            return;
        }

        gg.copyArea(0, 0, getWidth(), imgHeight, 0, -vertical * av.charHeight);

        int ss = av.startSeq;
        int es = av.endSeq;
        int transY = 0;

        if (vertical > 0) // scroll down
        {
            ss = es - vertical;

            if (ss < av.startSeq)
            { // ie scrolling too fast, more than a page at a time
                ss = av.startSeq;
            }
            else
            {
                transY = imgHeight - (vertical * av.charHeight);
            }
        }
        else if (vertical < 0)
        {
            es = ss - vertical;

            if (es > av.endSeq)
            {
                es = av.endSeq;
            }
        }

        gg.translate(0, transY);

        drawIds(ss, es);

        gg.translate(0, -transY);

        fastPaint = true;
        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     */
    public void paintComponent(Graphics g)
    {
        g.setColor(Color.white);
        g.fillRect(0, 0, getWidth(), getHeight());

        if (fastPaint)
        {
            fastPaint = false;
            g.drawImage(image, 0, 0, this);

            return;
        }

        imgHeight = getHeight();
        imgHeight -= (imgHeight % av.charHeight);

        if (imgHeight < 1)
        {
            return;
        }

        image = new BufferedImage(getWidth(), imgHeight,
                BufferedImage.TYPE_INT_RGB);
        gg = (Graphics2D) image.getGraphics();

        //Fill in the background
        gg.setColor(Color.white);
        gg.fillRect(0, 0, getWidth(), imgHeight);
        gg.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);

        Font italic = new Font(av.getFont().getName(), Font.ITALIC,
                av.getFont().getSize());
        gg.setFont(italic);

        drawIds(av.getStartSeq(), av.endSeq);

        g.drawImage(image, 0, 0, this);
    }

    /**
     * DOCUMENT ME!
     *
     * @param starty DOCUMENT ME!
     * @param endy DOCUMENT ME!
     */
    void drawIds(int starty, int endy)
    {
        Color currentColor = Color.white;
        Color currentTextColor = Color.black;

        if (av.getWrapAlignment())
        {
            int rowSize = av.getEndRes() - av.getStartRes();

            // Draw the rest of the panels
            for (int ypos = 2 * av.charHeight, row = av.startRes;
                    (ypos <= getHeight()) && (row < av.alignment.getWidth());
                    ypos += av.chunkHeight, row += rowSize)
            {
                for (int i = starty; i < av.alignment.getHeight(); i++)
                {
                    SequenceI s = av.alignment.getSequenceAt(i);
                    drawIdString(gg, s, i, 0, ypos);
                }
            }
        }
        else
        {
            //Now draw the id strings
            for (int i = starty; i < endy; i++)
            {
                // Selected sequence colours
                if ((searchResults != null) &&
                        searchResults.contains(av.alignment.getSequenceAt(i)))
                {
                    gg.setColor(Color.black);
                    currentColor = Color.black;
                    currentTextColor = Color.white;
                }
                else if ((av.getSelectionGroup() != null) &&
                        av.getSelectionGroup().sequences.contains(
                            av.alignment.getSequenceAt(i)))
                {
                    currentColor = Color.lightGray;
                    currentTextColor = Color.black;
                }
                else
                {
                    currentColor = av.alignment.getSequenceAt(i).getColor();
                    currentTextColor = Color.black;
                }

                gg.setColor(currentColor);

                gg.fillRect(0, (i - starty) * av.charHeight, getWidth(),
                    av.charHeight);

                gg.setColor(currentTextColor);

                String string = av.alignment.getSequenceAt(i).getName();

                if (av.getShowFullId())
                {
                    string = av.alignment.getSequenceAt(i).getDisplayId();
                }

                gg.drawString(string, 0,
                    (((i - starty) * av.charHeight) + av.charHeight) -
                    (av.charHeight / 5));
            }

            // add a border
            gg.setColor(Color.white);
            gg.fillRect(getWidth() - 4, 0, 4, getHeight());
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param found DOCUMENT ME!
     */
    public void setHighlighted(java.util.Vector found)
    {
        searchResults = found;
        repaint();
    }
}
